@@ -9,7 +9,17 @@ Created on 4 Mar 2021
 """
 
 import unittest
-from pynmeagps import NMEAMessage, GET, SET, POLL, NMEAMessageError
+from datetime import datetime
+from pynmeagps import (
+    NMEAMessage,
+    NMEAReader,
+    GET,
+    SET,
+    POLL,
+    NMEAMessageError,
+    VALCKSUM,
+    VALMSGID,
+)
 
 
 class FillTest(unittest.TestCase):
@@ -28,48 +38,38 @@ class FillTest(unittest.TestCase):
             payload=["5327.04319", "S", "00214.41396", "E", "223232.00", "A", "A"],
         )
         self.assertEqual(str(res), EXPECTED_RESULT)
+        res2 = NMEAReader.parse(res.serialize())
+        self.assertEqual(str(res2), EXPECTED_RESULT)
 
     def testFill_GNGLL_HP(self):  # test GET constructor in high precision mode
-        EXPECTED_RESULT = "<NMEA(GNGLL, lat=43.123456789, NS=N, lon=-2.987654321, EW=W, time=22:32:32, status=A, posMode=A)>"
-        EXPECTED_PAYLOAD = ["4307.4074073", "N", "00259.2592593", "W", "", "A", "A"]
+        EXPECTED_RESULT = "<NMEA(GNGLL, lat=43.123456789, NS=N, lon=-2.987654321, EW=W, time=16:29:24.123456, status=A, posMode=A)>"
+        EXPECTED_PAYLOAD = [
+            "4307.4074073",
+            "N",
+            "00259.2592593",
+            "W",
+            "162924.12",
+            "A",
+            "A",
+        ]
         res = NMEAMessage(
             "GN",
             "GLL",
             GET,
             lat=43.123456789,
-            NS="N",
-            lon=2.987654321,
-            EW="W",
-            time="22:32:32",
+            lon=-2.987654321,
+            time=datetime(2023, 11, 22, 16, 29, 24, 123456).time(),
             status="A",
             posMode="A",
             hpnmeamode=1,
         )
         self.assertEqual(str(res), EXPECTED_RESULT)
         self.assertEqual(res.payload, EXPECTED_PAYLOAD)
+        res2 = NMEAReader.parse(res.serialize())
+        print(res.serialize())
+        print(res2.serialize())
 
     def testFill_GNGLL_SP(self):  # test GET constructor in standard precision mode
-        EXPECTED_RESULT = "<NMEA(GNGLL, lat=43.123456789, NS=N, lon=-2.987654321, EW=W, time=22:32:32, status=A, posMode=A)>"
-        EXPECTED_PAYLOAD = ["4307.40741", "N", "00259.25926", "W", "", "A", "A"]
-        res = NMEAMessage(
-            "GN",
-            "GLL",
-            GET,
-            lat=43.123456789,
-            NS="N",
-            lon=2.987654321,
-            EW="W",
-            time="22:32:32",
-            status="A",
-            posMode="A",
-            hpnmeamode=0,
-        )
-        self.assertEqual(str(res), EXPECTED_RESULT)
-        self.assertEqual(res.payload, EXPECTED_PAYLOAD)
-
-    def testFill_GNGLL_NSEW1(
-        self,
-    ):  # derive NS or EW values
         EXPECTED_RESULT = "<NMEA(GNGLL, lat=43.123456789, NS=N, lon=-2.987654321, EW=W, time=22:32:32, status=A, posMode=A)>"
         EXPECTED_PAYLOAD = ["4307.40741", "N", "00259.25926", "W", "", "A", "A"]
         res = NMEAMessage(
@@ -86,17 +86,35 @@ class FillTest(unittest.TestCase):
         self.assertEqual(str(res), EXPECTED_RESULT)
         self.assertEqual(res.payload, EXPECTED_PAYLOAD)
 
+    def testFill_GNGLL_NSEW1(
+        self,
+    ):  # derive lat/lon sign from NS/EW values
+        EXPECTED_RESULT = "<NMEA(GNGLL, lat=-43.123456789, NS=S, lon=-2.987654321, EW=W, time=22:32:32, status=A, posMode=A)>"
+        EXPECTED_PAYLOAD = ["4307.40741", "S", "00259.25926", "W", "", "A", "A"]
+        res = NMEAMessage(
+            "GN",
+            "GLL",
+            GET,
+            lat=-43.123456789,
+            lon=-2.987654321,
+            time="22:32:32",
+            status="A",
+            posMode="A",
+            hpnmeamode=0,
+        )
+        self.assertEqual(str(res), EXPECTED_RESULT)
+        self.assertEqual(res.payload, EXPECTED_PAYLOAD)
+
     def testFill_GNGLL_NSEW2(
         self,
-    ):  # derive NS or EW values
-        EXPECTED_RESULT = "<NMEA(GNGLL, lat=-43.123456789, NS=S, lon=2.987654321, EW=E, time=22:32:32, status=A, posMode=A)>"
-        EXPECTED_PAYLOAD = ["4307.40741", "S", "00259.25926", "E", "", "A", "A"]
+    ):  # derive lat/lon sign from NS/EW values
+        EXPECTED_RESULT = "<NMEA(GNGLL, lat=43.123456789, NS=N, lon=2.987654321, EW=E, time=22:32:32, status=A, posMode=A)>"
+        EXPECTED_PAYLOAD = ["4307.40741", "N", "00259.25926", "E", "", "A", "A"]
         res = NMEAMessage(
             "GN",
             "GLL",
             GET,
             lat=43.123456789,
-            NS="S",
             lon=2.987654321,
             time="22:32:32",
             status="A",
@@ -106,6 +124,25 @@ class FillTest(unittest.TestCase):
         self.assertEqual(str(res), EXPECTED_RESULT)
         self.assertEqual(res.payload, EXPECTED_PAYLOAD)
 
+    def testFill_GNGLL_NSEW3(
+        self,
+    ):  # derive lat/lon sign from NS/EW values
+        EXPECTED_RESULT = "<NMEA(GNGLL, lat=43.123456789, NS=N, lon=-2.987654321, EW=W, time=22:32:32, status=A, posMode=A)>"
+        EXPECTED_PAYLOAD = ["4307.40741", "N", "00259.25926", "W", "", "A", "A"]
+        res = NMEAMessage(
+            "GN",
+            "GLL",
+            GET,
+            lat=43.123456789,
+            lon=-2.987654321,
+            time="22:32:32",
+            status="A",
+            posMode="A",
+            hpnmeamode=0,
+        )
+        self.assertEqual(str(res), EXPECTED_RESULT)
+        self.assertEqual(res.payload, EXPECTED_PAYLOAD)
+
     def testFill_GNGLLUPD(self):  # test that NMEAMessage is immutable after init
         EXPECTED_ERROR = (
             "Object is immutable. Updates to lon not permitted after initialisation."
@@ -163,21 +200,32 @@ class FillTest(unittest.TestCase):
     def testFill_UNKNOWN(self):  # test GET constructor with unknown msgId
         EXPECTED_ERROR = "Unknown msgID GNXXX, msgmode GET."
         with self.assertRaises(NMEAMessageError) as context:
-            NMEAMessage("GN", "XXX", GET, payload=[0, 0, 0])
+            NMEAMessage("GN", "XXX", GET, payload=[0, 0, 0], validate=VALMSGID)
         self.assertTrue(EXPECTED_ERROR in str(context.exception))
 
     def testFill_UNKNOWN2(self):  # test GET constructor with unknown talker
         EXPECTED_ERROR = "Unknown talker XX."
         with self.assertRaises(NMEAMessageError) as context:
-            NMEAMessage("XX", "XXX", GET, payload=[0, 0, 0])
+            NMEAMessage(
+                "XX", "XXX", GET, payload=[0, 0, 0], validate=VALCKSUM | VALMSGID
+            )
         self.assertTrue(EXPECTED_ERROR in str(context.exception))
 
-    def testFill_UNKNOWN3(self):  # test GET constructor with unknown UBX msgid
+    def testFill_UNKNOWN3(self):  # test GET constructor with unknown talker
+        EXPECTED_RESULT = "<NMEA(XXXXX, NOMINAL, field_01=0, field_02=0, field_03=0)>"
+        res = NMEAMessage("XX", "XXX", GET, payload=[0, 0, 0], validate=VALCKSUM)
+        self.assertEqual(str(res), EXPECTED_RESULT)
+
+    def testFill_UNKNOWN4(self):  # test GET constructor with unknown UBX msgid
         EXPECTED_ERROR = "Unknown msgID UBX08 msgmode GET."
         with self.assertRaises(NMEAMessageError) as context:
-            NMEAMessage("GN", "UBX", GET, payload=["08", 0, 0])
+            NMEAMessage("GN", "UBX", GET, payload=["08", 0, 0], validate=VALMSGID)
         self.assertTrue(EXPECTED_ERROR in str(context.exception))
 
+    def testFill_UNKNOWN5(self):  # test GET constructor with unknown msgId
+        EXPECTED_ERROR = "Unknown msgID GNXXX, msgmode GET."
+        msg = NMEAMessage("GN", "XXX", GET, payload=[0, 0, 0], validate=VALCKSUM)
+
 
 if __name__ == "__main__":
     # import sys;sys.argv = ['', 'Test.testName']
