@@ -10,7 +10,15 @@ Created on 4 Mar 2021
 """
 
 import unittest
-from pynmeagps import NMEAReader, NMEAParseError, SET, VALCKSUM, VALMSGID
+from pynmeagps import (
+    NMEAReader,
+    NMEAMessageError,
+    NMEAParseError,
+    ERR_RAISE,
+    SET,
+    VALCKSUM,
+    VALMSGID,
+)
 
 
 class ParseTest(unittest.TestCase):
@@ -84,8 +92,9 @@ class ParseTest(unittest.TestCase):
     def testParseNK2(
         self,
     ):  # unknown message identifier with validate VALCKSUM only - should just be ignored.
+        EXPECTED_RESULT = "<NMEA(GNXXX, NOMINAL, field_01=5327.04319, field_02=S, field_03=00214.41396, field_04=E, field_05=223232.00, field_06=A, field_07=A)>"
         res = NMEAReader.parse(self.messageNK, validate=VALCKSUM)
-        self.assertEqual(res, None)
+        self.assertEqual(str(res), EXPECTED_RESULT)
 
     def testParseBADMODE(self):  # invalid mode setting
         EXPECTED_ERROR = "Invalid parse mode 4 - must be 0, 1 or 2."
