@@ -8,45 +8,62 @@ Created on 3 Oct 2020
 :author: semuadmin
 """
 
-import unittest
 import datetime
+import os
+import unittest
+
 from pynmeagps import (
-    NMEAReader,
     NMEAMessage,
     NMEAMessageError,
+    NMEAReader,
     NMEATypeError,
+    NMEA_MSGIDS,
+    NMEA_MSGIDS_PROP,
+    NMEA_PAYLOADS_GET,
+    NMEA_PAYLOADS_POLL,
+    NMEA_PAYLOADS_SET,
 )
-from pynmeagps.nmeatypes_core import GET, POLL
 from pynmeagps.nmeahelpers import (
-    int2hexstr,
-    get_parts,
-    get_content,
+    area,
+    bearing,
     calc_checksum,
-    isvalid_cksum,
+    date2str,
+    date2utc,
+    ddd2dmm,
     deg2dmm,
     deg2dms,
     dmm2ddd,
-    ddd2dmm,
-    date2utc,
-    date2str,
-    time2str,
-    time2utc,
-    knots2spd,
-    msgdesc,
-    haversine,
-    bearing,
+    dms2deg,
     ecef2llh,
-    llh2ecef,
+    hex2str,
+    generate_checksum,
+    get_gpswnotow,
+    get_parts,
+    haversine,
+    knots2spd,
     latlon2dmm,
     latlon2dms,
+    llh2ecef,
     llh2iso6709,
-    get_gpswnotow,
+    msgdesc,
+    planar,
+    time2str,
+    time2utc,
+)
+from pynmeagps.nmeatypes_core import GET, POLL
+from pynmeagps.nmeatypes_decodes import (
+    GNSSLIST,
+    FIXTYPE_GGA,
+    FMI_STATUS,
+    SIGNALID,
+    SYSTEMID,
 )
 
 
 class StaticTest(unittest.TestCase):
     def setUp(self):
         self.maxDiff = None
+        dirname = os.path.dirname(__file__)
         self.messageCRAP = "$GNRMC,,%$£"
         self.messageBLANK = "$GNRMC,,A,,N,,W,0.046,,,,,A,V*0F"
         self.messageGLL = "$GNGLL,5327.04319,S,00214.41396,E,223232.00,A,A*68\r\n"
@@ -55,25 +72,33 @@ class StaticTest(unittest.TestCase):
         self.msgGLL = NMEAReader.parse(self.messageGLL)
         self.msgPUBX00 = NMEAReader.parse(self.messagePUBX)
         self.msgGNQ = NMEAMessage("EI", "GNQ", POLL, msgId="RMC")
+        self.streamNMEA2 = open(os.path.join(dirname, "pygpsdata-nmea2.log"), "rb")
 
     def tearDown(self):
-        pass
+        self.streamNMEA2.close()
 
     # *******************************************
     # Helper methods
     # *******************************************
 
-    def testInt2Hex(self):
-        res = int2hexstr(15)
-        self.assertEqual(res, "0F")
-        res = int2hexstr(104)
-        self.assertEqual(res, "68")
+    def testMissingDefs(self):  # sanity check for missing payload definitions
+        for msg in NMEA_MSGIDS:
+            self.assertTrue(
+                msg in NMEA_PAYLOADS_GET.keys()
+                or msg in NMEA_PAYLOADS_POLL
+                or msg in NMEA_PAYLOADS_SET
+            )
+
+    def testMissingCodes(self):  # sanity check for missing sentence IDs
+        for msg in NMEA_PAYLOADS_GET:
+            self.assertTrue(msg in NMEA_MSGIDS.keys() or msg in NMEA_MSGIDS_PROP.keys())
 
     def testGetParts(self):
         res = get_parts(self.messageGLL)
         self.assertEqual(
             res,
             (
+                "GNGLL,5327.04319,S,00214.41396,E,223232.00,A,A",
                 "GN",
                 "GLL",
                 ["5327.04319", "S", "00214.41396", "E", "223232.00", "A", "A"],
@@ -81,37 +106,45 @@ class StaticTest(unittest.TestCase):
             ),
         )
 
+    def testGenChecksum(self):  # test generation of new checksum
+        res = generate_checksum(
+            "GN", "GLL", ["5327.04319", "S", "00214.41396", "E", "223232.00", "A", "A"]
+        )
+        self.assertEqual(res, "68")
+        res = generate_checksum(
+            "GN", "GLL", ["1234.04319", "N", "00056.41396", "E", "123232.00", "A", "A"]
+        )
+        self.assertEqual(res, "75")
+
     def testGetPartsCRAP(self):  # test badly formed NMEA message
         EXPECTED_ERROR = "Badly formed message $GNRMC,,%$£"
         with self.assertRaises(NMEAMessageError) as context:
             get_parts(self.messageCRAP)
         self.assertTrue(EXPECTED_ERROR in str(context.exception))
 
-    def testGetContent(self):
-        res = get_content(self.messageGLL)
-        self.assertEqual(res, "GNGLL,5327.04319,S,00214.41396,E,223232.00,A,A")
-
     def testCalcChecksum(self):
-        res = calc_checksum(self.messageGLL)
+        content, _, _, _, _ = get_parts(self.messageGLL)
+        res = calc_checksum(content)
         self.assertEqual(res, "68")
-        res = calc_checksum(self.messagePUBX)
+        content, _, _, _, _ = get_parts(self.messagePUBX)
+        res = calc_checksum(content)
         self.assertEqual(res, "69")
 
-    def testGoodChecksum(self):
-        res = isvalid_cksum(self.messageGLL)
-        self.assertEqual(res, True)
-
-    def testBadChecksum(self):
-        res = isvalid_cksum(self.messageBADCK)
-        self.assertEqual(res, False)
-
     def testDMM2DDD(self):
-        res = dmm2ddd("5314.12345", "LA")
+        res = dmm2ddd("5314.12345")
         self.assertEqual(res, 53.2353908333)
-        res = dmm2ddd("00214.12345", "LN")
+        res = dmm2ddd("02348.3822990")
+        self.assertEqual(res, 23.80637165)
+        res = dmm2ddd("00234.3822990")
+        self.assertEqual(res, 2.5730383167)
+        res = dmm2ddd("00214.12345")
         self.assertEqual(res, 2.2353908333)
-        res = dmm2ddd("12825.12344", "LN")
+        res = dmm2ddd("12825.12344")
         self.assertEqual(res, 128.418724)
+        res = dmm2ddd("1282512344")
+        self.assertEqual(res, "")
+        res = dmm2ddd("2345.x5678")
+        self.assertEqual(res, "")
 
     def testDDD2DMM(self):
         res = ddd2dmm(3.75000, "LA")
@@ -137,11 +170,51 @@ class StaticTest(unittest.TestCase):
         res = ddd2dmm("", "LN", True)
         self.assertEqual(res, "")
 
+    def testDMS2DEG(self):
+        res = dms2deg("51°20′45.6″N")
+        self.assertEqual(res, 51.346)
+        self.assertEqual(deg2dms(res, "LA"), "51°20′45.6″N")
+        res = dms2deg("51°30′0.0″N")
+        self.assertEqual(res, 51.5)
+        self.assertEqual(deg2dms(res, "LA"), "51°30′0.0″N")
+        res = dms2deg("51°30.00′")
+        self.assertEqual(res, 51.5)
+        self.assertEqual(deg2dms(res, "LA"), "51°30′0.0″N")
+        res = dms2deg("2°20′45.6″W")
+        self.assertEqual(res, -2.346)
+        self.assertEqual(deg2dms(res, "LN"), "2°20′45.6″W")
+        res = dms2deg("33° 45′ 0.0″S")
+        self.assertEqual(res, -33.75)
+        self.assertEqual(deg2dms(res, "LA"), "33°45′0.0″S")
+        res = dms2deg("33°45′N")
+        self.assertEqual(res, 33.75)
+        self.assertEqual(deg2dms(res, "LN"), "33°45′0.0″E")
+        res = dms2deg("33°45.3564′N")
+        self.assertEqual(res, 33.75594)
+        self.assertEqual(deg2dms(res, "LA"), "33°45′21.384″N")
+        res = dms2deg("28°S")
+        self.assertEqual(res, -28.0)
+        self.assertEqual(deg2dms(res, "LA"), "28°0′0.0″S")
+        res = dms2deg("45.1234°")
+        self.assertEqual(res, 45.1234)
+        with self.assertRaises(ValueError) as context:
+            res = dms2deg("51°°20′45.6″")
+        with self.assertRaises(ValueError) as context:
+            res = dms2deg("51°20′45.6″Z")
+        with self.assertRaises(ValueError) as context:
+            res = dms2deg("51°20′45.6″34.25`N")
+        with self.assertRaises(ValueError) as context:
+            res = dms2deg("xx°yy′zz.6″N")
+
     def testDate2UTC(self):
         res = date2utc("")
         self.assertEqual(res, "")
         res = date2utc("120320")
         self.assertEqual(res, datetime.date(2020, 3, 12))
+        res = date2utc("031220", "DM")
+        self.assertEqual(res, datetime.date(2020, 3, 12))
+        res = date2utc("12032020", "DTL")
+        self.assertEqual(res, datetime.date(2020, 3, 12))
 
     def testTime2UTC(self):
         res = time2utc("")
@@ -158,6 +231,10 @@ class StaticTest(unittest.TestCase):
     def testDate2str(self):
         res = date2str(datetime.date(2021, 3, 7))
         self.assertEqual(res, "070321")
+        res = date2str(datetime.date(2021, 3, 7), "DTL")
+        self.assertEqual(res, "07032021")
+        res = date2str(datetime.date(2021, 3, 7), "DM")
+        self.assertEqual(res, "030721")
         res = date2str("wsdfasdf")
         self.assertEqual(res, "")
 
@@ -223,6 +300,11 @@ class StaticTest(unittest.TestCase):
         res = self.msgGNQ.msgmode
         self.assertEqual(res, POLL)
 
+    def testdatastream(self):  # test datastream getter
+        EXPECTED_RESULT = "<class '_io.BufferedReader'>"
+        res = str(type(NMEAReader(self.streamNMEA2).datastream))
+        self.assertEqual(res, EXPECTED_RESULT)
+
     def testPayloadS(self):
         res = self.msgGLL.payload
         self.assertEqual(
@@ -457,6 +539,17 @@ class StaticTest(unittest.TestCase):
         res = haversine(53.45, -2.14, 53.451, -2.141)
         self.assertAlmostEqual(res, 0.1296, 3)
 
+    def testplanar(self):  # test planar
+        res = planar(53, 2, 53.000001, 2.000001)
+        # print(res)
+        self.assertAlmostEqual(res, 0.12992378701316823, 7)
+        res = planar(53, 2, 53.00001, 2.00001)
+        # print(res)
+        self.assertAlmostEqual(res, 1.299237873027932, 7)
+        res = planar(53, 2, 53.0001, 2.0001)
+        # print(res)
+        self.assertAlmostEqual(res, 12.992378723687828, 7)
+
     def testbearing(self):
         res = bearing(51.23, -2.41, 53.205, -2.34)
         self.assertAlmostEqual(res, 1.216362703824359, 4)
@@ -467,6 +560,16 @@ class StaticTest(unittest.TestCase):
         res = bearing(-12.645, 34.867, -34.1745, 48.27846)
         self.assertAlmostEqual(res, 152.70835788275326, 4)
 
+    def testarea(self):
+        res = area(51.23, -2.41, 53.205, -2.34)
+        self.assertAlmostEqual(res, 1049.5657, 4)
+        res = area(53.48280729, -2.24225376, 53.46814647, -2.20192543)
+        self.assertAlmostEqual(res, 4.3606, 4)
+        res = area(53.69865772, -2.68269539, 53.22939103, -1.39218885)
+        self.assertAlmostEqual(res, 4467.6282, 4)
+        res = area(-12.645, 34.867, -34.1745, 48.27846)
+        self.assertAlmostEqual(res, 3264291.8230, 4)
+
     def testgpsweek(self):
         dats = [
             (2023, 1, 1),
@@ -488,6 +591,22 @@ class StaticTest(unittest.TestCase):
             y, m, d = dat
             self.assertEqual(get_gpswnotow(datetime.datetime(y, m, d)), vals[i])
 
+    def testhex2str(self):
+        hex = 0x1234ABCD
+        self.assertEqual(hex2str(hex, 8), "1234ABCD")
+        hex = 0x123B
+        self.assertEqual(hex2str(hex, 8), "0000123B")
+        self.assertEqual(hex2str(hex, 6), "00123B")
+        self.assertEqual(hex2str(hex), "123B")
+
+    def testdecodes(self):
+
+        self.assertEqual(GNSSLIST[0], "GPS")
+        self.assertEqual(FIXTYPE_GGA[1], "2D")
+        self.assertEqual(FMI_STATUS[2], ("Ready", "Filter convergence completion flag"))
+        self.assertEqual(SIGNALID[("1", "5")], "GPS L2 CM")
+        self.assertEqual(SYSTEMID["4"], "Beidou")
+
 
 if __name__ == "__main__":
     # import sys;sys.argv = ['', 'Test.testName']
